<?php
namespace SabaiApps\Directories\Component\Entity\DisplayElement;

use SabaiApps\Directories\Application;
use SabaiApps\Directories\Exception;
use SabaiApps\Directories\Component\Entity\Model\Bundle;

class TaxonomyFieldDisplayElement extends FieldDisplayElement
{
    protected $_taxonomy;

    public function __construct(Application $application, $name, $taxonomy)
    {
        parent::__construct($application, $name);
        $this->_taxonomy = $taxonomy;
    }

    protected function _doGetField($bundle)
    {
        if (!$taxonomy_bundle = $this->_application->Entity_Bundle($this->_taxonomy, $bundle->component, $bundle->group)) {
            throw new Exception\RuntimeException('Invalid taxonomy bundle');
        }
        $field_name = substr($this->_name, 17); // remove entity_tax_field_ part
        return $this->_application->Entity_Field($taxonomy_bundle, $field_name);
    }
    
    protected function _displayElementInfo(Bundle $bundle)
    {
        if (!$taxonomy_bundle = $this->_application->Entity_Bundle($this->_taxonomy, $bundle->component, $bundle->group)) {
            throw new Exception\RuntimeException('Invalid taxonomy bundle: ' . $this->_taxonomy);
        }
        $field = $this->_getField($bundle);
        
        return [
            'type' => 'taxonomy',
            'label' => $taxonomy_bundle->getLabel('singular') . ' - ' . $field->getFieldLabel(),
            'description' => sprintf(__('Field name: %s', 'directories'), $field->getFieldName()),
            'class' => 'drts-field-type-' . str_replace('_', '-', $field->getFieldType()) . ' drts-field-name-' . str_replace('_', '-', $field->getFieldName()),
            'default_settings' => array(
                'label' => 'none',
                'label_custom' => null,
                'label_icon' => null,
                'label_icon_size' => null,
                'renderer' => null,
                'renderer_settings' => [],
            ),
            'icon' => $this->_application->Field_Type($field->getFieldType())->fieldTypeInfo('icon'),
            'headingable' => false,
            'cacheable' => $this->_application->Field_Type($field->getFieldType())->fieldTypeInfo('cacheable'),
            'designable' => ['margin', 'font'],
        ];
    }

    public function displayElementRender(Bundle $bundle, array $element, $var)
    {
        if (!$entity = $var->getSingleFieldValue($this->_taxonomy)) return;

        $this->_application->Entity_Field_load($entity);
        return parent::displayElementRender($bundle, $element, $entity);
    }
    
    public function displayElementTitle(Bundle $bundle, array $element)
    {
        $field = $this->_getField($bundle);
        return $this->_application->Display_ElementLabelSettingsForm_label(
            $element['settings'],
            null,
            $this->_application->Entity_Bundle($this->_taxonomy, $bundle->component, $bundle->group)->getLabel('singular') . ' - ' . $field->getFieldLabel()
        );
    }
}
